C$$$   SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    M_ALTPRS    GET SFC PRESSURE FROM ALT SETTING
C
C   PRGMMR:  BENJAMIN, STAN ORG: ERL/PROFS      DATE: 93-01-12
C
C ABSTRACT:  DETERMINE SURFACE PRESSURE FROM ALTIMETER SETTING
C            AND ELEVATION.
C
C PROGRAM HISTORY LOG:
C    86-08-19      D. BAKER        ORIGINAL VERSION
C    01-07-20      M. BARTH        TURNED INTO A FUNCTION FOR MADIS
C
C USAGE:           M_ALTPRS = M_ALTPRS(ALT,ELEV)
C
C   INPUT ARGUMENT LIST:
C     ALT      - REAL     ALTIMETER SETTING IN MB
C     ELEV     - REAL     STATION ELEVATION IN METERS
C   OUTPUT ARGUMENT LIST:
C     M_ALTPRS - REAL     SURFACE PRESSURE IN MB
C
C REMARKS: NONE
C
C       Real    Alt,    ! Altimeter setting in mb
C    1          Elev,   ! Elevation in meters
C    1          Pres,   ! Surface pressure - output (IN MB)
C    1          const,  ! = R * GAMMA_STD / G
C    1          mslp,   ! Sea-level pressure for Std. Atmos.
C    1          lapse,  ! = Std. Atmos. lapse rate = 6.5 K/1000 m
C    1          tstd,   ! = Temp at sea-level in Std. Atmos. = 288.15K
C    1          FACTOR  ! Factor to account for variation of
C                          Temp. from Std. Atmos. to temp valid at Alt.
C                          setting
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN-77 + EXTENSIONS
C   MACHINE: UNIX
C
C$$$


      REAL FUNCTION M_ALTPRS(ALT,ELEV)

      INCLUDE 'IMPLICIT'

      REAL CONST
      REAL MSLP
      REAL LAPSE
      REAL TSTD
      REAL EXP
      PARAMETER ( CONST = 0.190284 )
      PARAMETER ( MSLP = 1013.25 )
      PARAMETER ( LAPSE = 0.0065 )
      PARAMETER ( TSTD = 288.15 )
      PARAMETER ( EXP = 1.0/CONST)


C     ARGUMENTS
      REAL  ALT
      REAL ELEV

C     LOCAL
      REAL FACTOR

C     ACCOUNT FOR VARIATION OF TSTD FROM TEMPERATURE VALID AT
C     PRESSURE OF ALTIMETER SETTING

      FACTOR = (MSLP/ALT)**CONST

C     CALCULATE SURFACE PRESSURE

      M_ALTPRS = ALT * (1.0-(LAPSE*ELEV*FACTOR/TSTD))**EXP

C     TO ACCOUNT FOR HEIGHT OF ALTIMETER IN COCKPIT ABOVE RUNWAY (3 M)
C     (I THINK), ADD 0.3 MB TO SFC PRESSURE

      M_ALTPRS = M_ALTPRS + 0.3

      RETURN
      END
C**---------------------------------------------------------------------
C**REQUIRED STANDARD FSL DISCLAIMER (NOVEMBER 2000)
C**---------------------------------------------------------------------
C**  OPEN SOURCE LICENSE/DISCLAIMER,  FORECAST SYSTEMS LABORATORY
C**  NOAA/OAR/FSL, 325 BROADWAY BOULDER, CO 80305
C**
C**  THIS SOFTWARE IS DISTRIBUTED UNDER THE OPEN SOURCE DEFINITION, 
C**  WHICH MAY BE FOUND AT http://www.opensource.org/osd.html.
C**
C**  IN PARTICULAR, REDISTRIBUTION AND USE IN SOURCE AND BINARY FORMS,
C**  WITH OR WITHOUT MODIFICATION, ARE PERMITTED PROVIDED THAT THE 
C**  FOLLOWING CONDITIONS ARE MET:
C**
C**  - REDISTRIBUTIONS OF SOURCE CODE MUST RETAIN THIS NOTICE, THIS LIST
C**    OF CONDITIONS AND THE FOLLOWING DISCLAIMER.
C**
C**  - REDISTRIBUTIONS IN BINARY FORM MUST PROVIDE ACCESS TO THIS 
C**    NOTICE, THIS LIST OF CONDITIONS AND THE FOLLOWING DISCLAIMER, AND
C**    THE UNDERLYING SOURCE CODE.
C**
C**  - ALL MODIFICATIONS TO THIS SOFTWARE MUST BE CLEARLY DOCUMENTED, 
C**    AND ARE SOLELY THE RESPONSIBILITY OF THE AGENT MAKING THE 
C**    MODIFICATIONS.
C**
C**  - IF SIGNIFICANT MODIFICATIONS OR ENHANCEMENTS ARE MADE TO THIS
C**    SOFTWARE, THE FSL SOFTWARE POLICY MANAGER 
C**    (softwaremgr.fsl@noaa.gov) BE NOTIFIED.
C**
C**  THIS SOFTWARE AND ITS DOCUMENTATION ARE IN THE PUBLIC DOMAIN AND 
C**  ARE FURNISHED "AS IS."  THE AUTHORS, THE UNITED STATES GOVERNMENT,
C**  ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO 
C**  WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE
C**  AND DOCUMENTATION FOR ANY PURPOSE.  THEY ASSUME NO RESPONSIBILITY 
C**  (1) FOR THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO 
C**  PROVIDE TECHNICAL SUPPORT TO USERS.
C**---------------------------------------------------------------------
