C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM:    MGETWISDOM  GET WISDOM OBS & QC INFO FOR ONE VARIABLE
C   PRGMMR: L. BENJAMIN      ORG: FSL        DATE: 08-07-20
C
C ABSTRACT: READ WISDOM OBS & QC INFO FOR ALL STATIONS FOR ONE
C   VARIABLE AND TIME.
C
C PROGRAM HISTORY LOG:
C   08-07-20  L. BENJAMIN ORIGINAL VERSION
C   08-08-15  L. BENJAMIN ADDED MISS_P AS MISSING DATA ARGUMENT FOR
C                         M_ZTOPSA.
C
C USAGE:    CALL MGETWISDOM (ATIME,VCODE,NSTA,NOBS,OB,QCDESC,QCAPP,
C                            QCRES,STATUS)
C
C   INPUT ARGUMENT LIST:
C     ATIME    - C       YYJJJHHMM OF THE DATA TO BE READ
C     VCODE    - C       VARIABLE CODE NAME
C
C   OUTPUT ARGUMENT LIST:
C     NSTA     - I       NUMBER OF STATION RECORDS RETURNED
C     NOBS     - I       NUMBER OF NON-MISSING OBS (<= NSTA)
C     OB       - RA      OBSERVATIONS IN M/K/S UNITS
C     QCDESC   - CA      QC DATA DESCRIPTORS
C     QCAPP    - IA      QC APPLIED WORD (SHOWS CHECKS APPLIED)
C     QCRES    - IA      QC RESULTS WORD (GOOD/BAD FOR EACH CHECK)
C     STATUS   - I       0 - SUCCESS, OR ERROR CODE
C
C   INPUT FILES:
C     MADIS WISDOM NETCDF FILES FOR ATIME
C
C SUBPROGRAMS CALLED:
C     UNIQUE:    MERROR, MWISDOMWIND, MGTNUM, MGTCHR, MQCFLTR, MGETVCN,
C                MGENVCN, M_ZTOPSA
C
C REMARKS:
C   - THE CALLER MUST CALL MINIT AND MWISDOMSTA BEFORE CALLING THIS
C     ROUTINE.  MWISDOMSTA SHOULD BE CALLED FIRST WHENEVER ACCESS TO
C     A NEW TIME IS DESIRED.  IT STORES INFO IN THE STRCCOM COMMON
C     THAT IS USED BY THIS ROUTINE TO FIGURE OUT WHICH RECORDS ARE
C     NEEDED ON NETCDF FILES OPENED BY MWISDOMSTA.
C   - DOUBLE & CHARACTER DATA TYPES ARE NOT HANDLED BY THIS ROUTINE.
C   - THE CALLER SHOULD DIMENSION ARRAYS TO MATCH MXWISDOM_P IN
C     madiscon.f.
C   - SEE madistmp.f FOR AN EXPLANATION OF HOW TEMPORARY STORAGE
C     IS HANDLED (ALL ARRAYS NAMED *TMP*).
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN-77 + EXTENSIONS
C   MACHINE:  UNIX
C$$$

      SUBROUTINE MGETWISDOM(ATIME,VCODE,NSTA,NOBS,OB,QCDESC,QCAPP,QCRES,
     1                      STATUS)

C     ------------------------------------------------------------------
C     INCLUDES AND COMMONS
C     ------------------------------------------------------------------
      INCLUDE 'IMPLICIT'
      INCLUDE 'MADISCON'
      INCLUDE 'MADISCOM'
      INCLUDE 'STRCCOM'
      INCLUDE 'MADISTMP'

C     ------------------------------------------------------------------
C     ARGUMENTS
C     ------------------------------------------------------------------
      CHARACTER*9 ATIME
      CHARACTER*(*) VCODE
      INTEGER NSTA,NOBS,QCAPP(*),QCRES(*),STATUS
      REAL OB(*)
      CHARACTER*1 QCDESC(*)

C     ------------------------------------------------------------------
C     LOCAL
C     ------------------------------------------------------------------
      INTEGER I,J,FNUM,DSN,LEN1REC,DUMMY,VCN,SPT,ORIG_VCN
      REAL OBTMP(MXWISDOM_P*4),QCATMP(MXWISDOM_P*4),QCRTMP(MXWISDOM_P*4)
      REAL M_ZTOPSA
      CHARACTER*1 QCDTMP(MXWISDOM_P*4),VCT,CDUMMY
      LOGICAL PROC_FILE(WISDOM_MXSTRF)

C     ------------------------------------------------------------------
C     TEMPORARY STORAGE DEFINITIONS
C     ------------------------------------------------------------------
      EQUIVALENCE(OBTMP,TMP_L1_N1)
      EQUIVALENCE(QCATMP,TMP_L1_N2)
      EQUIVALENCE(QCRTMP,TMP_L1_N3)
      EQUIVALENCE(QCDTMP,TMP_L1_C1)

C     ------------------------------------------------------------------
C     IF THE WISDOM SUBSYSTEM HASN'T BEEN INITIALIZED, OR IF THE 
C     REQUESTED TIME DOESN'T MATCH THE TIME LAST PROCESSED BY 
C     MWISDOMSTA, OR IF THE VCODE IS CLEARLY ILLEGAL, RETURN WITH ERROR.
C     ------------------------------------------------------------------
      IF(WISDOM_INIT.NE.INIT_P)THEN
         STATUS = SELSUB_NOTINIT_P
         CALL MERROR('MGETWISDOM: SELECTED SUBSYSTEM NOT INITIALIZED'//
     1               ' - WISDOM')
         GO TO 900
      ELSE IF(ATIME.NE.LAST_WISDOM_ATIME.OR.ATIME.EQ.'         ')THEN
         STATUS = TIME_NOTINIT_P
         CALL MERROR('MGETWISDOM: TIME NOT INITIALIZED BY MWISDOMSTA')
         GO TO 900
      ELSE 
         CALL MGETVCN(VCODE,ORIG_VCN,VCT,STATUS)
         IF(STATUS.NE.SUCCESS_P)GO TO 900
      ENDIF

C     ------------------------------------------------------------------
C     SET THE CURRENT DATA TYPE.
C     ------------------------------------------------------------------
      CDTYPE = WISDOM_DS_P

C     ------------------------------------------------------------------
C     TRANSLATE THE CALLER'S VCN INTO THE GENERIC VERSION.  THIS IS USED
C     TO ALLOW THE CALLER TO SELECT ONLY SPECIFIC WIND PRODUCTS (E.G.
C     IR ONLY).  FOR MOST OF THIS ROUTINE (AND ITS SUBORDINATES) WE'LL 
C     DEAL WITH THE GENERIC VERSION TO MAKE THINGS SIMPLER, THEN DEAL 
C     WITH THE CALLER'S DESIRED SUBSET WHEN IT COMES TIME TO RETURN THE 
C     ACTUAL DATA.
C     ------------------------------------------------------------------
      CALL MGENVCN(ORIG_VCN,WISDOM_DS_P,CDBTYPE,VCN,SPT)

C     ------------------------------------------------------------------
C     FILL IN MISSING FLAGS FOR ALL POSSIBLE RECORDS IN CALLER'S ARRAY,
C     INDICATE THAT WE HAVEN'T PROCESSED ANY FILES YET.
C     THEN LOOP THROUGH THE FILES THAT ARE NEEDED AND PUT IN REAL DATA
C     WHERE AVAILABLE.
C     ------------------------------------------------------------------
      NSTA = WISDOM_LNUM
      IF(NSTA.EQ.0)THEN
         STATUS = NO_DATA_P
         GO TO 900
      ENDIF

      FNUM = 0

      DO 1 I = 1, NSTA
         OB(I) = MISS_P
         OBTMP(I) = MISS_P
         QCDESC(I) = 'Z'
         QCAPP(I) = 0
         QCRES(I) = 0
 1    CONTINUE

      DO 2 I = 1, WISDOM_MXSTRF
         PROC_FILE(I) = .FALSE.
 2    CONTINUE

C     ------------------------------------------------------------------
C     NOTE THAT WE LOOP THROUGH WISDOM_FNUM TO SEE WHICH FILES ARE 
C     NEEDED AND NOT THROUGH WISDOM_NFILES.  THIS IS DONE AS JUST 
C     BECAUSE A FILE GOT READ DOESN'T MEAN ANY OF ITS RECORDS ACTUALLY 
C     WERE EXTRACTED WHEN ALL THE FILTERING WAS DONE IN MWISDOMSTA.  
C     THEREFORE, WE'LL ONLY PROCESS EACH FILE WITH RECORDS IN USE.
C     ------------------------------------------------------------------
      DO 100 I = 1, NSTA

C     ------------------------------------------------------------------
C     ALREADY HANDLED THIS FILE.
C     ------------------------------------------------------------------
         IF(PROC_FILE(WISDOM_FNUM(I)))GO TO 100

         FNUM = WISDOM_FNUM(I)
         PROC_FILE(FNUM) = .TRUE.
         DSN = MAX_SFCDSET_P + MAX_UAPTDSET_P + MAX_HYDRODSET_P +
     1        MAX_SATWDSET_P + MAX_SATSDSET_P + MAX_SATRDSET_P + 
     2        MAX_SNOWDSET_P +
     1         WISDOM_SOURCE(FNUM)

C     ------------------------------------------------------------------
C     THE VARIABLE'S NOT ASSOCIATED WITH THIS SOURCE DATASET.
C     ------------------------------------------------------------------
         IF(VLEN(VCN,DSN).EQ.0)GO TO 100

C     ------------------------------------------------------------------
C     THE VARIABLE'S DOUBLE OR CHARACTER DATA TYPE:  THIS WILL BE
C     TRUE FOR ANY DATASET FOR WHICH THIS VARIABLE APPLIES, AND IS
C     AN ILLEGAL OPTION.
C     ------------------------------------------------------------------
         IF(VDTYPE(VCN,DSN).EQ.'d'.OR.VDTYPE(VCN,DSN).EQ.'c')THEN
            STATUS = INV_DTYPE_P
            CALL MERROR('MGETWISDOM: INVALID VARIABLE DATA TYPE')
            GO TO 900
         ENDIF

C     ------------------------------------------------------------------
C     IF THE VARIABLE NEEDS TO BE CREATED FROM OTHER VARIABLES, CALL
C     A SUBROUTINE TO FILL IN THE LOCAL ARRAYS.  OTHERWISE, READ THE
C     VARIABLE, AND IF AVAILABLE, QC INFO.  IF THERE'S AN ERROR,
C     THE SUBROUTINES WILL HAVE SET STATUS & CALLED MERROR.
C     ------------------------------------------------------------------
         IF(VCN.EQ.WISDOMU_VN_P.OR.VCN.EQ.WISDOMV_VN_P.OR.
     1      VCN.EQ.WISDOMFF_VN_P.OR.VCN.EQ.WISDOMDD_VN_P)THEN

            CALL MWISDOMWIND(VCN,DSN,FNUM,OBTMP,QCDTMP,QCATMP,QCRTMP,
     1                       STATUS)
            IF(STATUS.NE.SUCCESS_P)GO TO 900

         ELSE

            CALL MGTNUM(VNAME(VCN,DSN)(1:VLEN(VCN,DSN)),
     1           WISDOM_NCID(FNUM),WISDOM_RECDIM(FNUM),1,
     2           WISDOM_NRECS(FNUM),MISS_P,OBTMP,STATUS)
            IF(STATUS.NE.SUCCESS_P)GO TO 900

C     ASSUME QC OK, SET FOR DIFFERNET CONDITIONS
            QCSTATUS = EQCOK_P

            IF(VQC(VCN,DSN).EQ.'Q')THEN

               CALL MGTNUM(VNAME(VCN,DSN)(1:VLEN(VCN,DSN))//'QCA',
     1                     WISDOM_NCID(FNUM),WISDOM_RECDIM(FNUM),1,
     2                     WISDOM_NRECS(FNUM),MISS_P,QCATMP,STATUS)
               IF(STATUS.EQ.VAR_NOTINDB_P)THEN
                  QCSTATUS = EQCFAILED_P
                  STATUS = SUCCESS_P
                  DO J = 1, WISDOM_NRECS(FNUM)
                  
                     QCDTMP(J) = 'Z'
                     QCATMP(J) = 0
                     QCRTMP(J) = 0
                  ENDDO
               ELSE IF(STATUS.NE.SUCCESS_P)THEN
                  GO TO 900
               ELSE

                  CALL MGTNUM(VNAME(VCN,DSN)(1:VLEN(VCN,DSN))//'QCR',
     1                 WISDOM_NCID(FNUM),WISDOM_RECDIM(FNUM),1,
     2                 WISDOM_NRECS(FNUM),MISS_P,QCRTMP,STATUS)
                  IF(STATUS.NE.SUCCESS_P)GO TO 900
                  
                  CALL MGTCHR(VNAME(VCN,DSN)(1:VLEN(VCN,DSN))//'DD',
     1                 WISDOM_NCID(FNUM),WISDOM_RECDIM(FNUM),1,
     2                 WISDOM_NRECS(FNUM),QCDTMP,LEN1REC,STATUS)
                  IF(STATUS.NE.SUCCESS_P)GO TO 900                  
               ENDIF
            ELSE
               QCSTATUS = EQCFAILED_P

               DO 10 J = 1, WISDOM_NRECS(FNUM)

                  QCDTMP(J) = 'Z'
                  QCATMP(J) = 0
                  QCRTMP(J) = 0

 10            CONTINUE

            ENDIF
            IF(VCN.EQ.WISDOMP_VN_P)THEN

C     ------------------------------------------------------------------
C     IN ORDER TO GET PRESSURE, WE READ IN HEIGHT (JUST DID THIS) THEN
C     CONVERT TO GET PRESSURE, USE THE U.S. STANDARD ATMOSPHERE
C     CALCULATION, AND CONVERT FROM MB TO PASCALS.
C     ------------------------------------------------------------------

               DO J = 1, WISDOM_NRECS(FNUM)

                  IF(OBTMP(J).NE.MISS_P)
     1                 OBTMP(J) = M_ZTOPSA(OBTMP(J),MISS_P) * 100.

               ENDDO

            ENDIF

         ENDIF

C     ------------------------------------------------------------------
C     DO ANY REQUESTED QC FILTERING.  NOTE THAT THIS IS DONE INSIDE THE
C     DO 100 LOOP SO THAT MQCFLTR CAN HANDLE EACH WISDOM DATASET
C     INDIVIDUALLY.  THIS IS NEEDED TO COVER THE CASE OF QC INFO NOT
C     BEING AVAILABLE FOR ALL WISDOM DATASETS, BUT IS AVAILABLE FOR 
C     SOME.
C     ------------------------------------------------------------------
         CALL MQCFLTR(VCN,DSN,1,WISDOM_NRECS(FNUM),QCDTMP,DUMMY,DUMMY,
     1                OBTMP)

C     ------------------------------------------------------------------
C     EXTRACT DESIRED RECORDS INTO CALLER'S ARRAY.  NOTE THAT IF WE'RE
C     GETTING A SPECIFIC WIND PRODUCT TYPE, MAKE SURE WE HAVE THE TYPES
C     IN COMMON (IT'S THE "LEVEL TYPE" VARIABLE).
C     ------------------------------------------------------------------
         IF(SPT.GT.0)THEN

            DO 120 J = I, NSTA

               IF(WISDOM_FNUM(J).NE.FNUM)GO TO 120

               OB(J) = OBTMP(WISDOM_RNUM(J))
               QCDESC(J) = QCDTMP(WISDOM_RNUM(J))
               QCAPP(J) = QCATMP(WISDOM_RNUM(J))
               QCRES(J) = QCRTMP(WISDOM_RNUM(J))

 120        CONTINUE

         ELSE

            DO 130 J = I, NSTA

               IF(WISDOM_FNUM(J).NE.FNUM)GO TO 130

               OB(J) = OBTMP(WISDOM_RNUM(J))
               QCDESC(J) = QCDTMP(WISDOM_RNUM(J))
               QCAPP(J) = QCATMP(WISDOM_RNUM(J))
               QCRES(J) = QCRTMP(WISDOM_RNUM(J))

 130        CONTINUE

         ENDIF

 100  CONTINUE

 900  IF(STATUS.NE.SUCCESS_P)THEN

         NSTA = 0
         NOBS = 0

      ELSE

C     ------------------------------------------------------------------
C     COUNT UP THE NUMBER OF NON-MISSING OBS ACTUALLY RETURNED.
C     ------------------------------------------------------------------
         NOBS = 0

         DO 121 J = 1, NSTA
            IF(OB(J).NE.MISS_P)NOBS = NOBS + 1
 121     CONTINUE

         IF(NOBS.EQ.0)THEN
            NSTA = 0
            STATUS = NO_DATA_P
         ENDIF

      ENDIF

      RETURN
      END

C**---------------------------------------------------------------------
C**REQUIRED STANDARD FSL DISCLAIMER (NOVEMBER 2000)
C**---------------------------------------------------------------------
C**  OPEN SOURCE LICENSE/DISCLAIMER,  FORECAST SYSTEMS LABORATORY
C**  NOAA/OAR/FSL, 325 BROADWAY BOULDER, CO 80305
C**
C**  THIS SOFTWARE IS DISTRIBUTED UNDER THE OPEN SOURCE DEFINITION, 
C**  WHICH MAY BE FOUND AT http://www.opensource.org/osd.html.
C**
C**  IN PARTICULAR, REDISTRIBUTION AND USE IN SOURCE AND BINARY FORMS,
C**  WITH OR WITHOUT MODIFICATION, ARE PERMITTED PROVIDED THAT THE 
C**  FOLLOWING CONDITIONS ARE MET:
C**
C**  - REDISTRIBUTIONS OF SOURCE CODE MUST RETAIN THIS NOTICE, THIS LIST
C**    OF CONDITIONS AND THE FOLLOWING DISCLAIMER.
C**
C**  - REDISTRIBUTIONS IN BINARY FORM MUST PROVIDE ACCESS TO THIS 
C**    NOTICE, THIS LIST OF CONDITIONS AND THE FOLLOWING DISCLAIMER, AND
C**    THE UNDERLYING SOURCE CODE.
C**
C**  - ALL MODIFICATIONS TO THIS SOFTWARE MUST BE CLEARLY DOCUMENTED, 
C**    AND ARE SOLELY THE RESPONSIBILITY OF THE AGENT MAKING THE 
C**    MODIFICATIONS.
C**
C**  - IF SIGNIFICANT MODIFICATIONS OR ENHANCEMENTS ARE MADE TO THIS
C**    SOFTWARE, THE FSL SOFTWARE POLICY MANAGER 
C**    (softwaremgr.fsl@noaa.gov) BE NOTIFIED.
C**
C**  THIS SOFTWARE AND ITS DOCUMENTATION ARE IN THE PUBLIC DOMAIN AND 
C**  ARE FURNISHED "AS IS."  THE AUTHORS, THE UNITED STATES GOVERNMENT,
C**  ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO 
C**  WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE
C**  AND DOCUMENTATION FOR ANY PURPOSE.  THEY ASSUME NO RESPONSIBILITY 
C**  (1) FOR THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO 
C**  PROVIDE TECHNICAL SUPPORT TO USERS.
C**---------------------------------------------------------------------
